#include <mutek/startup.h>
#include <mutek/thread.h>
#include <mutek/printk.h>
#include <stdio.h>

#include <device/class/nfc.h>
#include <device/class/char.h>
#include <device/class/timer.h>
#include <device/class/pwm.h>

#define BACKLIGHT 12

static CONTEXT_ENTRY(nfc_main)
{
    error_t err;
    struct device_char_s lcd;
    struct device_timer_s timer;
    struct device_pwm_s pwm;
    dev_timer_delay_t sec;
    struct dev_pwm_config_s pwm_config;
    struct device_nfc_s nfc;
    struct dev_nfc_rq_s req;
    struct dev_nfc_peer_s peer;
    char buffer[64];

    pwm_config.freq.num = 16000;
    pwm_config.freq.denom = 1;
    pwm_config.duty.num = 0;
    pwm_config.duty.denom = 16;
    pwm_config.pol = DEV_PWM_POL_HIGH;

    req.peer = &peer;
    peer.side = DEV_NFC_PASSIVE_PICC;
    peer.protocol = DEV_NFC_14443A;

    // HIP fob
    //endian_le32_na_store(peer.uid, 0xdb70aa74);
    //peer.uid_size = 4;

    // MF1K + LF "B"
    //endian_le32_na_store(peer.uid, 0x0053a381);
    //peer.uid_size = 4;

    // MF1K clone "A"
    //endian_le32_na_store(peer.uid, 0x0053e3d7);
    //peer.uid_size = 4;

    // NTAG demo board
    //endian_le64_na_store(peer.uid, 0x803b7c3a520e04ull);
    //peer.uid_size = 7;

    // NTAG v6 board
    endian_le64_na_store(peer.uid, 0x8040c802c59d04ull);
    peer.uid_size = 7;
    peer.atqa = 0;

    err = device_get_accessor_by_path(&nfc.base, NULL, "/nfc0", DRIVER_CLASS_NFC);
    assert(!err);

    err = device_get_accessor_by_path(&lcd.base, NULL, "/lcd*", DRIVER_CLASS_CHAR);
    assert(!err);

    err = device_get_accessor_by_path(&pwm.base, NULL, "/pwm*", DRIVER_CLASS_PWM);
    assert(!err);

    err = device_get_accessor_by_path(&timer.base, NULL, "/rtc* /timer*", DRIVER_CLASS_TIMER);
    assert(!err);

    device_start(&pwm.base);
    dev_pwm_wait_config(&pwm, &pwm_config,
                        DEV_PWM_MASK_DUTY|DEV_PWM_MASK_FREQ|DEV_PWM_MASK_POL);
    dev_timer_init_sec(&timer, &sec, 0, 1, 10);

    size_t n;

    n = snprintf(buffer, sizeof(buffer), "\x1b[2J" "\x1b[0;0H" "Please put tag",
                 peer.uid, peer.uid_size, peer.sak, peer.atqa);
    dev_char_wait_op(&lcd, DEV_CHAR_WRITE_FLUSH, (void *)buffer, n);

    n = snprintf(buffer, sizeof(buffer), "\x1b[1;0H" "%P",
                 peer.uid, peer.uid_size);
    dev_char_wait_op(&lcd, DEV_CHAR_WRITE_FLUSH, (void *)buffer, n);

    while (1) {
      req.type = DEV_NFC_POWEROFF;
      dev_nfc_wait_request(&nfc, &req);

      req.type = DEV_NFC_SELECT;

      err = dev_nfc_wait_request(&nfc, &req);

      if (err == 0) {
        pwm_config.duty.num = 16;
        dev_pwm_wait_config(&pwm, &pwm_config, DEV_PWM_MASK_DUTY);
      } else {
        pwm_config.duty.num = 1;
        dev_pwm_wait_config(&pwm, &pwm_config, DEV_PWM_MASK_DUTY);
      }
    }
}

void app_start(void)
{
  struct thread_attr_s attr = {
    .stack_size = 2048,
  };

  thread_create(nfc_main, 0, &attr);
}
