#include <mutek/startup.h>
#include <mutek/thread.h>
#include <mutek/printk.h>
#include <stdio.h>

#include <device/class/nfc.h>
#include <device/class/char.h>
#include <device/class/timer.h>
#include <device/class/pwm.h>

#define BACKLIGHT 12

static CONTEXT_ENTRY(nfc_main)
{
  error_t err;
  struct device_char_s lcd;
  struct device_timer_s timer;
  struct device_pwm_s pwm;
  dev_timer_delay_t sec;
  struct dev_pwm_config_s pwm_config;
  struct device_nfc_s nfc;
  struct dev_nfc_rq_s req;
  struct dev_nfc_peer_s peer;
  char buffer[64];
  uint8_t data[64];

  pwm_config.freq.num = 16000;
  pwm_config.freq.denom = 1;
  pwm_config.duty.num = 0;
  pwm_config.duty.denom = 16;
  pwm_config.pol = DEV_PWM_POL_HIGH;

  req.peer = &peer;
  peer.side = DEV_NFC_PASSIVE_PICC;
  peer.protocol = DEV_NFC_14443A;

  err = device_get_accessor_by_path(&nfc.base, NULL, "/nfc0", DRIVER_CLASS_NFC);
  assert(!err);

  err = device_get_accessor_by_path(&lcd.base, NULL, "/lcd*", DRIVER_CLASS_CHAR);
  assert(!err);

  err = device_get_accessor_by_path(&pwm.base, NULL, "/pwm*", DRIVER_CLASS_PWM);
  assert(!err);

  err = device_get_accessor_by_path(&timer.base, NULL, "/rtc* /timer*", DRIVER_CLASS_TIMER);
  assert(!err);

  device_start(&pwm.base);
  dev_pwm_wait_config(&pwm, &pwm_config,
                      DEV_PWM_MASK_DUTY|DEV_PWM_MASK_FREQ|DEV_PWM_MASK_POL);
  dev_timer_init_sec(&timer, &sec, 0, 1, 1);

  size_t n;

  while (1) {
    n = snprintf(buffer, sizeof(buffer), "\x1b[2J" "\x1b[0;0H" "Type 2 reader",
                 peer.uid, peer.uid_size, peer.sak, peer.atqa);
    dev_char_wait_op(&lcd, DEV_CHAR_WRITE_FLUSH, (void *)buffer, n);

    req.type = DEV_NFC_POWEROFF;
    dev_nfc_wait_request(&nfc, &req);

    req.type = DEV_NFC_SELECT_ANY;
    err = dev_nfc_wait_request(&nfc, &req);

    if (err != 0)
      continue;

    /* if (peer.atqa != 0x0044 || peer.sak != 0x00) { */
    /*   n = snprintf(buffer, sizeof(buffer), "\x1b[2J" */
    /*                "\x1b[0;0H" "ATQA %04x SAK %02x" */
    /*                "\x1b[1;0H" "Not a type 2 tag !", */
    /*                peer.sak, peer.atqa); */
    /*   dev_char_wait_op(&lcd, DEV_CHAR_WRITE_FLUSH, (void *)buffer, n); */

    /*   dev_timer_wait_delay(&timer, sec, 0); */

    /*   continue; */
    /* } */

    size_t size = sizeof(data);
    err = dev_nfc_wait_transceive_std(&nfc, &peer,
                                      (const uint8_t []){0x30, 0x04}, 2,
                                      data, &size);
    if (err) {
      printk("transceive error %d\n", err);
      continue;
    }

    printk("transceive ok %d bytes %P\n", size, data, size);
    dev_timer_wait_delay(&timer, sec, 0);
  }
}

void app_start(void)
{
  struct thread_attr_s attr = {
    .stack_size = 2048,
  };

  thread_create(nfc_main, 0, &attr);
}
